/*---------------------------------------------------------------------------*/
/*           Copyright (c) 1996 LAL Orsay, IN2P3-CNRS (France).              */
/*                                                                           */
/* Redistribution and use in source and binary forms, with or without        */
/* modification, are permitted provided that the following conditions        */
/* are met:                                                                  */
/* 1. Redistributions of source code must retain the above copyright         */
/*    notice, this list of conditions and the following disclaimer.          */
/* 2. Redistributions in binary form must reproduce the above copyright      */
/*    notice, this list of conditions and the following disclaimer in the    */
/*    documentation and/or other materials provided with the distribution.   */
/* 3. All advertising materials mentioning features or use of this software  */
/*    must display the following acknowledgement:                            */
/*      This product includes software developed by the Computer Application */
/*      Development Group at LAL Orsay (Laboratoire de l'Accelerateur        */
/*      Linaire - IN2P3-CNRS).                                               */
/* 4. Neither the name of the Institute nor of the Laboratory may be used    */
/*    to endorse or promote products derived from this software without      */
/*    specific prior written permission.                                     */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED BY THE LAL AND CONTRIBUTORS ``AS IS'' AND       */
/* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE     */
/* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR        */
/* PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE LAL OR CONTRIBUTORS BE      */
/* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR       */
/* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF      */
/* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS  */
/* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN   */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)   */
/* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF    */
/* THE POSSIBILITY OF SUCH DAMAGE.                                           */
/*---------------------------------------------------------------------------*/
#ifndef __CSet_h__
#define __CSet_h__

#include <GC.h>

typedef struct _CArrayRec*    CArray;
typedef struct _CStackRec*    CStack;
typedef struct _CQueueRec*    CQueue;
typedef struct _CLListRec*    CLList;
typedef struct _CHasherRec*   CHasher;
typedef struct _CIteratorRec* CIterator;

typedef void  (*CMethod)   (void*);
typedef void* (*CValuator) (void* object);
typedef int   (*CCoder)    (void* object);
typedef int   (*CTester)   (void* object, void* other);

void CArrayStartup  ();
void CArrayCleanup  ();
void CArrayCheck  ();
CArray CArrayNew  ();
void CArrayErase  (CArray This);
int CArrayCount  (CArray This);
void CArrayAdd  (CArray This, void* object);
void CArrayRemove  (CArray This, void* object);
void* CArrayItem  (CArray This, CIterator it);
void CArrayRemoveAt  (CArray This, int index);
void* CArrayItemAt  (CArray This, int index);
void* CArrayFind  (CArray This, void* object);
CIterator CArrayGiveIterator  (CArray This);
CIterator CArrayGiveBackwardIterator  (CArray This);
CStack CStackNew  ();
void CStackErase  (CStack This);
int CStackCount  (CStack This);
void CStackPush  (CStack This, void* object);
void* CStackTop  (CStack This);
void* CStackPop  (CStack This);
CQueue CQueueNew  ();
void CQueueErase  (CQueue This);
int CQueueCount  (CQueue This);
void CQueuePush  (CQueue This, void* object);
void* CQueueTop  (CQueue This);
void* CQueuePop  (CQueue This);
CLList CLListNew  ();
void CLListErase  (CLList This);
int CLListCount  (CLList This);
void CLListAdd  (CLList This, void* object);
void CLListPrepend  (CLList This, void* object);
void* CLListFirst  (CLList This);
void* CLListLast  (CLList This);
void CLListRemove  (CLList This, void* object);
void* CLListFind  (CLList This, void* object);
CIterator CLListGiveIterator  (CLList This);
CIterator CLListGiveBackwardIterator  (CLList This);
CHasher CPtrHasherNew  (int size, CValuator valuator);
CHasher CStrHasherNew  (int size, CValuator valuator);
CHasher CLStrHasherNew  (int size, CValuator valuator);
CHasher CHasherNew  (int size,                    CValuator valuator,                    CCoder coder,                    CTester tester);
void CHasherErase  (CHasher This);
void CHasherAdd  (CHasher This, void* object);
void* CHasherFind  (CHasher This, void* value);
int CHasherFindPosition  (CHasher This, void* value,                         int* code, int* index);
void CHasherRemove  (CHasher This, void* object);
CIterator CHasherGiveIterator  (CHasher This);
int CIteratorFinished  (CIterator This);
void CIteratorFirst  (CIterator This);
void CIteratorForth  (CIterator This);
void CIteratorDelete  (CIterator This);
void* CIteratorItem  (CIterator This);
void CIteratorStop  (CIterator This);

#endif
